import React, { Fragment } from 'react'
import { get } from 'lodash'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'
import { required } from 'xpub-validators'
import { compose, setDisplayName, withHandlers, withProps } from 'recompose'
import {
  H2,
  Menu,
  Button,
  Spinner,
  Checkbox,
  TextField,
  ValidatedField,
} from '@pubsweet/ui'
import {
  Row,
  Text,
  Item,
  Label,
  IconButton,
  RowOverrideAlert,
  ItemOverrideAlert,
} from 'pubsweet-component-faraday-ui'

const AddUserForm = ({ roles, countries, titles }) => (
  <Fragment>
    <Row alignItems="baseline" mb={1} mt={1}>
      <Item mr={1} vertical>
        <Label required>Email</Label>
        <ValidatedField
          component={TextField}
          name="email"
          validate={[required]}
        />
      </Item>
      <ItemOverrideAlert ml={1} vertical>
        <Label required>Role</Label>
        <ValidatedField
          component={input => (
            <Menu options={roles} {...input} placeholder="Please select" />
          )}
          name="role"
          validate={[required]}
        />
      </ItemOverrideAlert>
    </Row>

    <Row mb={2}>
      <Item mr={1} vertical>
        <Label>First Name</Label>
        <ValidatedField component={TextField} name="firstName" />
      </Item>
      <Item ml={1} vertical>
        <Label>Last Name</Label>
        <ValidatedField component={TextField} name="lastName" />
      </Item>
    </Row>

    <RowOverrideAlert alignItems="center" mb={2}>
      <ItemOverrideAlert mr={1} vertical>
        <Label>Title</Label>
        <ValidatedField
          component={input => (
            <Menu options={titles} {...input} placeholder="Please select" />
          )}
          name="title"
        />
      </ItemOverrideAlert>
      <ItemOverrideAlert ml={1} vertical>
        <Label>Country</Label>
        <ValidatedField
          component={input => (
            <Menu options={countries} {...input} placeholder="Please select" />
          )}
          name="country"
        />
      </ItemOverrideAlert>
    </RowOverrideAlert>

    <Row mb={3}>
      <Item vertical>
        <Label>Affiliation</Label>
        <ValidatedField component={TextField} name="affiliation" />
      </Item>
    </Row>
  </Fragment>
)

const EditForm = ({ titles, countries }) => (
  <Fragment>
    <Row alignItems="center" mb={2} mt={1}>
      <Item mr={1} vertical>
        <Label>First Name</Label>
        <ValidatedField component={TextField} name="firstName" />
      </Item>
      <Item ml={1} vertical>
        <Label>Last Name</Label>
        <ValidatedField component={TextField} name="lastName" />
      </Item>
    </Row>

    <RowOverrideAlert alignItems="center" mb={2}>
      <ItemOverrideAlert mr={1} vertical>
        <Label>Title</Label>
        <ValidatedField
          component={input => (
            <Menu options={titles} {...input} placeholder="Please select" />
          )}
          name="title"
        />
      </ItemOverrideAlert>
      <ItemOverrideAlert ml={1} vertical>
        <Label>Country</Label>
        <ValidatedField
          component={input => (
            <Menu options={countries} {...input} placeholder="Please select" />
          )}
          name="country"
        />
      </ItemOverrideAlert>
    </RowOverrideAlert>

    <Row mb={2}>
      <Item vertical>
        <Label>Affiliation</Label>
        <ValidatedField component={TextField} name="affiliation" />
      </Item>
    </Row>
    <Row>
      <Item>
        <Label>Roles</Label>
      </Item>
    </Row>
    <RowOverrideAlert mb={3} mt={1}>
      <Item>
        <ValidatedField
          component={({ value, onChange }) => (
            <Checkbox
              checked={value}
              label="Editor in Chief"
              onChange={onChange}
              value={value}
            />
          )}
          name="editorInChief"
        />
      </Item>
      <Item>
        <ValidatedField
          component={({ value, onChange }) => (
            <Checkbox
              checked={value}
              label="Admin"
              onChange={onChange}
              value={value}
            />
          )}
          name="admin"
        />
      </Item>
      <Item>
        <ValidatedField
          component={({ value, onChange }) => (
            <Checkbox
              checked={value}
              label="Handling Editor"
              onChange={onChange}
              value={value}
            />
          )}
          name="handlingEditor"
        />
      </Item>
    </RowOverrideAlert>
  </Fragment>
)

const FormModal = ({
  edit,
  user,
  roles,
  title,
  titles,
  onClose,
  subtitle,
  onConfirm,
  countries,
  modalError,
  isFetching,
  handleSubmit,
  confirmText = 'OK',
  cancelText = 'Cancel',
}) => (
  <Root>
    <IconButton icon="x" onClick={onClose} right={5} secondary top={5} />
    <H2>{title}</H2>
    {edit && <Text secondary>{user.email}</Text>}
    {edit ? (
      <EditForm countries={countries} titles={titles} />
    ) : (
      <AddUserForm countries={countries} roles={roles} titles={titles} />
    )}
    {modalError && (
      <Row mb={1}>
        <Text error>{modalError}</Text>
      </Row>
    )}
    <Row>
      {isFetching ? (
        <Spinner size={3} />
      ) : (
        <Fragment>
          <Button onClick={onClose}>{cancelText}</Button>
          <Button onClick={handleSubmit} primary>
            {confirmText}
          </Button>
        </Fragment>
      )}
    </Row>
  </Root>
)

const setInitialRole = a => {
  if (get(a, 'handlingEditor')) {
    return 'handlingEditor'
  }
  if (get(a, 'editorInChief')) {
    return 'editorInChief'
  }
  return 'author'
}

export default compose(
  withHandlers({
    onConfirm: ({ onConfirm, ...props }) => () => {
      if (onConfirm && typeof onConfirm === 'function') {
        onConfirm(props)
      }
    },
    onClose: ({ onCancel, ...props }) => () => {
      if (onCancel && typeof onCancel === 'function') {
        onCancel(props)
      }
      props.hideModal()
    },
  }),
  withProps(({ user, edit }) => ({
    initialValues: {
      ...user,
      role: setInitialRole(user),
    },
    confirmText: edit ? 'EDIT USER' : 'SAVE USER',
  })),
  reduxForm({
    form: 'add-edit',
  }),
  setDisplayName('FormModal'),
)(FormModal)

// #region styles
const Root = styled.div`
  align-items: center;
  background: ${th('colorBackgroundHue')};
  border: ${th('borderWidth')} ${th('borderStyle')} transparent;
  border-radius: ${th('borderRadius')};
  box-shadow: ${th('boxShadow')};
  display: flex;
  flex-direction: column;
  position: relative;
  padding: calc(${th('gridUnit')} * 3);
  width: calc(${th('gridUnit')} * 60);

  ${H2} {
    margin: 0;
    text-align: center;
  }
`
// #endregion
